#!/usr/bin/env pike

void usage() {
    write(#"Usage:
-s, --server\t\tspacewalk server
-u, --username\t\tusername
-p, --password\t\tpassword
-n, --name\t\treturn name instead of label
-f, --file\t\tdefaults file
");
}

mapping config=([]);

mapping get_file_opts(string filename) {
    string file;
    if (Stdio.exist(filename))
        file=Stdio.read_file(filename);
    else
        return ([]);
    return Public.Tools.ConfFiles.Config.read(file);
}

int main(int argc, array argv) {
    array opts=Getopt.find_all_options(argv,aggregate(
        ({"server",Getopt.HAS_ARG,({"-s","--server"}) }),
        ({"username",Getopt.HAS_ARG,({"-u","--username"}) }),
        ({"password",Getopt.HAS_ARG,({"-p","--password"}) }),
        ({"name",Getopt.NO_ARG,({"-n","--name"}) }),
        ({"file",Getopt.HAS_ARG,({"-f","--file"}) }),
    ),1,1);
    string server="http://localhost/rpc/api",user="",pass="";
    string file="";
    int name=0;

    argv=Getopt.get_args(argv,1,1);
    foreach (opts; int ind; array opt) {
        switch (opt[0]) {
            case "file":
                config=get_file_opts(opt[1]);
                if (config && config->sw) {
                    if (config->sw->user)
                        user=config->sw->user;
                    if (config->sw->pass)
                        pass=config->sw->pass;
                    if (config->sw->host)
                        server=config->sw->host;
                }
                break;
            case "server":
                // ipa/cpr/ == /rpc/api backwards
                if (opt[1][..6]=="http://" || reverse(opt[1])[..7]=="ipa/cpr/")
                    server=opt[1];
                else
                    server="http://"+opt[1]+"/rpc/api";
                break;
            case "username":
                user=opt[1];
                break;
            case "password":
                pass=opt[1];
                break;
            case "name":
                name=opt[1];
                break;
        }
    }
    if (server[..6]!="http://" || reverse(server)[..7]!="ipa/cpr/")
        server="http://"+server+"/rpc/api";
    if (server=="")
        server="http://localhost/rpc/api";
    if (user=="" || pass=="") {
        usage();
        return 1;
    }
    object x=Public.Tools.Spacewalk.client(server);
    x->login(user,pass);
    array channels=x->list_all_channels(name);
//    array channels=x->base_channel_list();
//    foreach (channels; int ind; mixed p) {
//        array children=x->channel_children_list();
//    }
    write("channels:\n%O\n",channels);
    x->logout();
}
// vi:tabstop=4:expandtab:ai
