#!/usr/bin/env pike

mapping config=([]);

mapping get_file_opts(string filename) {
    string file;
    if (Stdio.exist(filename))
        file=Stdio.read_file(filename);
    else
        return ([]);
    return Public.Tools.ConfFiles.Config.read(file);
}

int main(int argc, array argv) {
    array opts=Getopt.find_all_options(argv,aggregate(
        ({"server",Getopt.HAS_ARG,({"-s","--server"}) }),
        ({"username",Getopt.HAS_ARG,({"-u","--username"}) }),
        ({"password",Getopt.HAS_ARG,({"-p","--password"}) }),
        ({"file",Getopt.HAS_ARG,({"-f","--file"}) }),
    ),1,1);

    string server="",user="",pass="",channel="";
    string file="";

    argv=Getopt.get_args(argv,1,1);
    foreach (opts; int ind; array opt) {
        switch (opt[0]) {
            case "file":
                config=get_file_opts(opt[1]);
                if (config && config->sw) {
                    if (config->sw->user)
                        user=config->sw->user;
                    if (config->sw->pass)
                        pass=config->sw->pass;
                    if (config->sw->host)
                        server=config->sw->host;
                }
                break;
            case "server":
                // ipa/cpr/ == /rpc/api backwards
                if (opt[1][..6]=="http://" && reverse(opt[1])[..7]=="ipa/cpr/")
                    server=opt[1];
                else
                    server="http://"+opt[1]+"/rpc/api";
                break;
            case "username":
                user=opt[1];
                break;
            case "password":
                pass=opt[1];
                break;
        }
    }
    if (server[..6]!="http://" || reverse(server)[..7]!="ipa/cpr/")
        server="http://"+server+"/rpc/api";
    if (server=="")
        server="http://localhost/rpc/api";
    if (user=="" || pass=="")
        return 1;
    object x=Public.Tools.Spacewalk.client(server);
    x->login(user,pass);

    mapping sys_rel_arch=([]);
    array systems=x->system_list();
    string arch;
    mapping d;
    foreach (systems; int ind; string sys) {
        d=x->system_getdetails(sys);
        arch=(array_sscanf(d->description,"%*sCPU Arch: %s")[0]/"-")[0];
        if (arch[..4]=="ia32e")
            arch="x86_64";
        sys_rel_arch[sys]=([ "release": d->release, "arch": arch ]);
    }
    foreach (sort(indices(sys_rel_arch)); int ind; string name)
        write("%s,%s,%s\n",name,sys_rel_arch[name]->release,sys_rel_arch[name]->arch);
}
// vi:tabstop=4:expandtab:ai
