#!/usr/bin/env pike

import Tools.Logging;

mapping config=([]);

mapping get_file_opts(string filename) {
    string file;
    if (Stdio.exist(filename))
        file=Stdio.read_file(filename);
    else
        return ([]);
    return Public.Tools.ConfFiles.Config.read(file);
}

int main(int argc, array argv) {
    array opts=Getopt.find_all_options(argv,aggregate(
        ({"server",Getopt.HAS_ARG,({"-s","--server"}) }),
        ({"username",Getopt.HAS_ARG,({"-u","--username"}) }),
        ({"password",Getopt.HAS_ARG,({"-p","--password"}) }),
        ({"file",Getopt.HAS_ARG,({"-f","--file"}) }),
        ({"package",Getopt.HAS_ARG,({"-P","--package"}) }),
        ({"system",Getopt.HAS_ARG,({"-S","--system"}) }),
        ({"all",Getopt.NO_ARG,({"-a","--all"}) }),
        ({"verbose",Getopt.NO_ARG,({"-v","--verbose"}) }),
    ),1,1);

    string server="",user="",pass="";
    string file="",package="",system="";
    int all_systems=0;
    int verbose=0;

    argv=Getopt.get_args(argv,1);
    foreach (opts; int ind; array opt) {
        switch (opt[0]) {
            case "file":
                config=get_file_opts(opt[1]);
                if (config && config->sw) {
                    if (config->sw->user)
                        user=config->sw->user;
                    if (config->sw->pass)
                        pass=config->sw->pass;
                    if (config->sw->host)
                        server=config->sw->host;
                }
                break;
            case "server":
                // ipa/cpr/ == /rpc/api backwards
                if (opt[1][..6]=="http://" || reverse(opt[1])[..7]=="ipa/cpr/")
                    server=opt[1];
                else
                    server="http://"+opt[1]+"/rpc/api";
                break;
            case "username":
                user=opt[1];
                break;
            case "password":
                pass=opt[1];
                break;
            case "system":
                system=opt[1];
                break;
            case "all":
                all_systems=(int)opt[1];
                break;
            case "package":
                package=opt[1];
                break;
            case "verbose":
                verbose=(int)opt[1];
                break;
        }
    }
    if (server[..6]!="http://" || reverse(server)[..7]!="ipa/cpr/")
        server="http://"+server+"/rpc/api";
    if (server=="")
        server="http://localhost/rpc/api";
    if (user=="" || pass=="" || (system=="" && all_systems==0) || package=="") {
        Log.error("Must have user, pass, package, and either system or --all");
        return 1;
    }
    object x=Public.Tools.Spacewalk.client(server);
    x->login(user,pass);

    if (all_systems) {
        foreach (x->system_list();; string sys) {
            if (verbose)
                Log.info("Scheduling install of %s on %s\n",package,sys);
            x->system_schedule_package_install(sys,package);
        }
    } else {
        if (verbose)
            Log.info("Scheduling install of %s on %s\n",package,system);
        x->system_schedule_package_install(system,package);
    }
}
// vi:tabstop=4:expandtab:ai

