import Tools.Logging;

Protocols.XMLRPC.Client x;
string url;
string user,pass;
string session;
mixed res;

constant SYSTEM_CACHE_TTL=3600;
constant PACKAGE_CACHE_TTL=86400;
constant ERRATA_CACHE_TTL=86400;
constant MINIMUM_API_VERSION=10.0;
array ENTITLEMENTS = ({ "provisioning_entitled",
    "enterprise_entitled",
    "monitoring_entitled",
    "virtualization_host",
    "virtualization_host_platform",
});

array SYSTEM_SEARCH_FIELDS = ({ "id","name","ip","hostname","device",
    "vendor","driver","uuid" });

mapping system_cache=0;
mapping package_cache=0;
mapping errata_cache=0;
mapping channel_cache=0;

void create(string loc, int|void loglevel) {
    url=loc;
    x=Protocols.XMLRPC.Client(url);
    init_caches();
    Log.set_level(loglevel);
}

void throw_fault(Protocols.XMLRPC.Fault r) {
    throw( ({ sprintf("%O",r), backtrace() }) );
}

mixed sw_api(string api, mixed ... args) {
    res=x[api](@args);
    if (objectp(res))
        throw_fault(res);
    return res[0];
}

void login(string username, string password) {
    user=username;
    pass=password;
    session=sw_api("auth.login",username,password);
    session=res[0];
}

void logout() {
    sw_api("auth.logout",session);
}

void init_caches() {
    system_cache=([]);
    package_cache=([]);
    errata_cache=([]);
    channel_cache=([]);
}

void update_system_cache() {
    array systems=sw_api("system.listSystems",session);
    system_cache=mkmapping(values(systems)->name,values(systems)->id);
}

void update_channel_cache() {
    array channels=sw_api("channel.listAllChannels",session);
    /*
    foreach (channels; int ind; mapping p) {
        array d=sw_api("channel.software.getDetails",session,p->label);
        channel_cache[p->label]=p-(<"label">)+d;
    }
    */
    channel_cache=mkmapping(channels->label,channels);
    foreach (indices(channel_cache); int ind; string label) {
        channel_cache[label]+=sw_api("channel.software.getDetails",session,label);
    }
}

mapping get_channel_details(string channel) {
    if (sizeof(channel_cache)==0) {
        update_channel_cache();
    }
    if (has_index(channel_cache,channel))
        return channel_cache[channel];
    return ([]);
}

array system_list() {
    if (sizeof(system_cache)==0) {
        update_system_cache();
    }
    return sort(indices(system_cache));
}

mapping system_getdetails(string system) {
    if (sizeof(system_cache)==0) {
        update_system_cache();
    }
    return sw_api("system.getDetails",session,system_cache[system]);
}

string system_get_subscribed_base_channel(string system) {
    if (sizeof(system_cache)==0) {
        update_system_cache();
    }
    return sw_api("system.getSubscribedBaseChannel",session,system_cache[system])->label;
}

array system_list_subscribed_child_channels(string system) {
    if (sizeof(system_cache)==0) {
        update_system_cache();
    }
    return sw_api("system.listSubscribedChildChannels",session,system_cache[system])->label;
}

int system_set_base_channel(string system, string channel) {
    if (sizeof(system_cache)==0) {
        update_system_cache();
    }
    return sw_api("system.setBaseChannel",session,system_cache[system],channel);
}

int system_set_child_channels(string system, array channels) {
    if (sizeof(system_cache)==0) {
        update_system_cache();
    }
    return sw_api("system.setChildChannels",session,system_cache[system],channels);
}

array list_all_channels(int|void only_name) {
    if (sizeof(channel_cache)==0) {
        update_channel_cache();
    }
    if (only_name) {
        return sort(values(channel_cache)["name"]);
//        return sort(values(map(channel_cache, lambda(mapping p) { return p->name; } )));
    }
    return sort(indices(channel_cache));
}

array channel_list_children(string label) {
    return sw_api("channel.software.listChildren",session,label)[*]["label"];
}

array system_reboot() {
}

string system_createpackageprofile() {
}

mixed softwarechannel_clone(string name, string label, string source_channel,
	string|void parent, object|void original_state, string|void gpg_url,
    string|void gpg_id, string|void gpg_fingerprint, string|void summary,
    string|void desc) {
    mapping details=([]);
    details["name"]=name;
    details["label"]=label;
    if (stringp(parent))
        details["parent_label"]=parent;
    if (stringp(gpg_url))
        details["gpg_url"]=gpg_url;
    if (stringp(gpg_id))
        details["gpg_id"]=gpg_id;
    if (stringp(gpg_fingerprint))
        details["gpg_fingerprint"]=gpg_fingerprint;
    if (stringp(desc))
        details["description"]=desc;
    object orig=Val.false;
    if (objectp(original_state) && original_state->is_val_true)
        orig=Val.true;
    if (stringp(summary))
        details["summary"]=summary;
    else
        details["summary"]=details["label"];
    mixed r1=sw_api("channel.software.clone",session,source_channel,details,orig);
    return r1;
    
}

array channel_list_subscribed_systems(string channel) {
    if (sizeof(channel_cache)==0) {
        update_channel_cache();
    }
    return sw_api("channel.software.listSubscribedSystems",session,channel)[*]["name"];
}

int delete_channel(string channel) {
    return sw_api("channel.software.delete",session,channel);
}

int system_schedule_package_install(string system, string package) {
    if (sizeof(system_cache)==0) {
        update_system_cache();
    }
//    if (sizeof(package_cache)==0) {
//        update_package_cache();
//    }
    int package_id=find_latest_package(system,package);
    return sw_api("system.schedulePackageInstall",session,system_cache[system],({ package_id }),Calendar.Hour());

}

int find_latest_package(string system, string package) {
    array p,channels;
    mapping pm;
    string version="",release="",name="";
    int ppid=0;

    mapping m=system_getdetails(system);
    channels=({ system_get_subscribed_base_channel(system) });
    channels+=system_list_subscribed_child_channels(system);

    p=filter(sw_api("packages.search.advanced",session,"name:"+package),lambda(mapping n) { if (n->name==package) return 1; return 0; } );
    array package_ids=p[*]["id"];
    foreach (package_ids;; int pid) {
        pm=sw_api("packages.getDetails",session,pid);
        foreach (pm["providing_channels"];; string pc) {
            if (search(channels,pc)>=0) {
                if (compare_versions(pm->version,version)) {
                    version=pm->version;
                    release=pm->release;
                    ppid=pid;
                    Log.info("package: %s  version: %s  release: %s  id: %d",package,version,release,ppid);
                } else {
                    if (compare_versions(pm->release,release)) {
                        release=pm->release;
                        ppid=pid;
                        Log.info("package: %s  version: %s  release: %s  id: %d",package,version,release,ppid);
                    }
                }
            }
        }
    }
    Log.info("Final:");
    Log.info("package: %s  version: %s  release: %s  id: %d",package,version,release,ppid);

    return ppid;
}

int compare_versions(string v1, string v2) {
    array va1=v1/".";
    array va2=v2/".";
    for (int i=0; i<min(sizeof(va1),sizeof(va2)); i++) {
        if ((int)va1[i]>(int)va2[i])
            return 1;
        if ((int)va2[i]>(int)va1[i])
            return 0;
    }
    return 0;
}
// vi:tabstop=4:expandtab:ai
